@@ -164,6 +164,8 @@ static void InitGroups(Translator *tr)
 
 		if (p[0] == RULE_LETTERGP2) {
 			ix = p[1] - 'A';
+			if (ix < 0)
+				ix += 256;
 			p += 2;
 			if ((ix >= 0) && (ix < N_LETTER_GROUPS))
 				tr->letterGroups[ix] = p;
@@ -208,8 +210,8 @@ int LoadDictionary(Translator *tr, const char *name, int no_error)
 	unsigned int size;
 	char fname[sizeof(path_home)+20];
 
-	strcpy(dictionary_name, name); // currently loaded dictionary name
-	strcpy(tr->dictionary_name, name);
+	strncpy(dictionary_name, name, 40); // currently loaded dictionary name
+	strncpy(tr->dictionary_name, name, 40);
 
 	// Load a pronunciation data file into memory
 	// bytes 0-3:  offset to rules data
@@ -658,9 +660,27 @@ const char *GetTranslatedPhonemeString(int phoneme_mode)
 
 static int IsLetterGroup(Translator *tr, char *word, int group, int pre)
 {
-	// match the word against a list of utf-8 strings
-	char *p;
-	char *w;
+	/* Match the word against a list of utf-8 strings.
+	 *
+	 * How this works:
+	 *
+	 *       +-+
+	 *       |c|<-(tr->letterGroups[group])
+	 *       |0|
+	 *   *p->|c|<-len+              +-+
+	 *       |s|<----+              |a|<-(Actual word to be tested)
+	 *       |0|            *word-> |t|<-*w=word-len+1 (for pre-rule)
+	 *       |~|                    |a|<-*w=word       (for post-rule)
+	 *       |7|                    |s|
+	 *       +-+                    +-+
+	 *
+	 *     7=RULE_GROUP_END
+	 *     0=null terminator
+	 *     pre==1 — pre-rule
+	 *     pre==0 — post-rule
+	 */
+	char *p; // group counter
+	char *w; // word counter
 	int len = 0;
 
 	p = tr->letterGroups[group];
@@ -673,17 +693,33 @@ static int IsLetterGroup(Translator *tr, char *word, int group, int pre)
 			w = word - len + 1;
 		} else
 			w = word;
+
+		// If no character is allowed in group
+		// at the start (for pre-rule) or end (post-rule)
+		// of the checked letter in the word, return true.
+		if (*p == '~' && *w == ' ') // word end checked because of comment below
+			return 1;
+		/* TODO: Need to investigate why word end mark _ doesn't work properly
+		 * for post rule somewhere in MatchRule() function. or e.g.:
+		 *
+		 * .L01 ~ b c
+		 * .group a
+		 *  _L01) a       i  // this works
+		 *        a (L01_ u  // this doesn't work
+		 */
+
 		while ((*p == *w) && (*w != 0)) {
 			w++;
 			p++;
 		}
-		if (*p == 0) {
+		if (*p == 0) { // Matched the current group.
 			if (pre)
 				return len;
-			return w-word; // matched a complete string
+			return w - word;
 		}
 
-		while (*p++ != 0) // skip to end of string
+		// No match, so skip the rest of this group.
+		while (*p++ != 0)
 			;
 	}
 	return 0;
@@ -1537,6 +1573,7 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 	unsigned char rb;     // current instuction from rule
 	unsigned char letter; // current letter from input word, single byte
 	int letter_w;         // current letter, wide character
+	int last_letter_w;    // last letter, wide character
 	int letter_xbytes;    // number of extra bytes of multibyte character (num bytes - 1)
 	unsigned char last_letter;
 
@@ -1595,6 +1632,7 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 		match_type = 0;
 		consumed = 0;
 		letter = 0;
+		letter_w = 0;
 		distance_right = -6; // used to reduce points for matches further away the current letter
 		distance_left = -2;
 		check_atstart = 0;
@@ -1694,11 +1732,12 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 					failed = 1;
 				break;
 			case RULE_POST:
-				// continue moving fowards
+				// continue moving forwards
 				distance_right += 6;
 				if (distance_right > 18)
 					distance_right = 19;
 				last_letter = letter;
+				last_letter_w = letter_w;
 				letter_xbytes = utf8_in(&letter_w, post_ptr)-1;
 				letter = *post_ptr++;
 
@@ -1717,6 +1756,8 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 					break;
 				case RULE_LETTERGP2: // match against a list of utf-8 strings
 					letter_group = *rule++ - 'A';
+					if (letter_group < 0)
+						letter_group += 256;
 					if ((n_bytes = IsLetterGroup(tr, post_ptr-1, letter_group, 0)) > 0) {
 						add_points = (20-distance_right);
 						post_ptr += (n_bytes-1);
@@ -1750,7 +1791,7 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 						failed = 1;
 					break;
 				case RULE_DOUBLE:
-					if (letter == last_letter)
+					if (letter_w == last_letter_w)
 						add_points = (21-distance_right);
 					else
 						failed = 1;
@@ -1855,7 +1896,7 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 					break;
 				case RULE_DEC_SCORE:
 					add_points = -20; // force an decrease in points
-					break;            // TODO schould check why '<' is shown as '!' in rules trace
+					break;
 				case RULE_DEL_FWD:
 					// find the next 'e' in the word and replace by 'E'
 					for (p = *word + group_length; p < post_ptr; p++) {
@@ -1903,6 +1944,7 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 					distance_left = 19;
 
 				last_letter = *pre_ptr;
+				utf8_in(&last_letter_w, pre_ptr);
 				pre_ptr--;
 				letter_xbytes = utf8_in2(&letter_w, pre_ptr, 1)-1;
 				letter = *pre_ptr;
@@ -1921,7 +1963,9 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 						failed = 1;
 					break;
 				case RULE_LETTERGP2: // match against a list of utf-8 strings
-					letter_group = *rule++ - 'A';
+					letter_group = *rule++ - 'A'; // substracting 'A' makes letter_group equal to number in .Lxx definition
+					if(letter_group<0)
+						letter_group += 256;
 					if ((n_bytes = IsLetterGroup(tr, pre_ptr, letter_group, 1)) > 0) {
 						add_points = (20-distance_right);
 						pre_ptr -= (n_bytes-1);
@@ -1936,7 +1980,7 @@ static void MatchRule(Translator *tr, char *word[], char *word_start, int group_
 						failed = 1;
 					break;
 				case RULE_DOUBLE:
-					if (letter == last_letter)
+					if (letter_w == last_letter_w)
 						add_points = (21-distance_left);
 					else
 						failed = 1;
